/* --TT
 * This file only define one funtion check(), which check the information from
 * monitor and decide whether trigger the Detector/
 */

#include "include/check.h"

#define SYS_DURATION 		2
#define IOWAIT_DURATION		3
#define SWP_DURATION		3

/* use duration array to count continuously occur times */
static int cpu_cond_dura1[CPU_NR];  /* for sys, usr, idle */
static int cpu_cond_dura2[CPU_NR];  /* for sys, usr */
static int cpu_cond_dura3[CPU_NR];  /* for iowait */
static int mem_swp_cond_dura;       /* for pswpout_per_sec */

/**
 * If the hang rule is satisfied, return 1, or return 0.
 * return a code include error information, most suit 30-core CPU
 */
int check() {
  unsigned int rc = 0;
  int i;
  /* check each CPU info */
  for (i = 0; i < CPU_NR; ++i) {
    if (ci[i].sys >= 95.0) {
      rc |= (1 << i);
    } else if (ci[i].sys >= 87.0) {
      if (ci[i].usr < ci[i].idle + ci[i].iowait) {
        if (++cpu_cond_dura1[i] >= SYS_DURATION)
          rc |= (1 << i);
      } else {
        cpu_cond_dura1[i] = 0;
      }
    } else if (ci[i].sys >= 50.0) {
      if (ci[i].usr <= 4.0) {
        if (++cpu_cond_dura2[i] >= SYS_DURATION)
          rc |= (1 << i);
      } else {
        cpu_cond_dura2[i] = 0;
      }
    }
    if (ci[i].iowait >= 50.0) {
      if (++cpu_cond_dura3[i] >= IOWAIT_DURATION || ci[i].iowait >= 60.0)
        rc |= (1 << i);
    } else {
      cpu_cond_dura3[i] = 0;
    }
  }

  /* check proc */
  if (pi.run_proc_num > 10 * CPU_NR || pi.cs_per_sec < 350) /* depend on CPU */
    rc |= PROC_ERROR;

  /* check memory */
  /* may should be relaxed condition, but enough to discover memory bomb */
  if (mi.pswpout_per_sec >= 3000.0) {
    if (++mem_swp_cond_dura >= SWP_DURATION || mi.pswpout_per_sec >= 5000.0)
      rc |= MEM_ERROR;
  } else {
    mem_swp_cond_dura = 0;
  }

  /* check disk I/O, empty */

  return rc;
}

