#include <unistd.h>
#include <stdlib.h>

#include "include/info.h"
#include "include/check.h"
#include "include/alert.h"

#define __USE_GNU
#include <sched.h>	// for cpu_set_t, CPU_SET(), sched_setaffinity()

static int alert_interval;

void get() {
  get_cpu_info();	/* CPU */
  get_proc_info();	/* process */
  get_swap_info();	/* memory */
  get_paging_info();	/* memory */
  get_blk_info();     	/* I/O block */
  get_memstatis_info();	/* memory */
  get_memutil_info();	/* memory */
  get_swaputil_info();	/* memory */
  get_task_info();	/* process */	
  get_devIO_info();	/* dev I/O */
}

void monitoring() {
  JUMP_A_LINE;
  JUMP_A_LINE;
  unsigned int rc;
  for (;;) {
    get();
    if (rc = check()) {
      if (!alert_interval)
        alert(rc);
      if (++alert_interval == 5)
        alert_interval = 0;
    }
    wakeup_detector();
  }
}

void start() {
  pid_t pid;
  int fd[2];
  if (pipe(fd) < 0) {
    perror("Monitor: pipe error");
  }
  if ((pid = fork()) < 0) {
    perror("Monitor: fork error");
  } else if (pid > 0) {  /* parent */
    close(fd[1]);        /* parent read from pipe */
    dup2(fd[0], STDIN_FILENO);  /* set pipe to stdio */
    monitoring();
  } else {               /* child */
    close(fd[0]);        /* child write to pipe */
    dup2(fd[1], STDOUT_FILENO); /* set pipe to stdout */
    char* const argv[] = { /* sar -u ALL -P ALL -qw -WBSRr -d 1 */
        "/usr/bin/sar",
        "-u", "ALL", 	/* all CPU */
        "-P", "ALL",	/* each CPU */
        "-qw", 		/* process */
        "-WBSRr", 	/* memory */
        "-bd", 		/* IO */
        "1", 
        NULL };
    execve("/usr/bin/sar", argv, NULL);
  }
}

void kill_handler(int sig) {
  system("rmmod detector.ko");
  system("rmmod recover.ko");
  close_alert();
  exit(0);
}

int main(void) {
  nice(-20);
  /* bind Monitor run on CPU0 */
  cpu_set_t mask;
  CPU_SET(0, &mask);
  sched_setaffinity(getpid(), sizeof(mask), &mask);

  init_alert();
  (void) signal(SIGINT, kill_handler);
  alert_interval = 0;
  start();
  return 0;
}

